# Daniel Capilla (anyWARE AG Mainz)
# Date: 2009-06-19
# Version 0.7
# - Initial stable beta
# Version 0.7.1
# - stable release
# - added error handling and output
# - restructuring script
# Syntax: .\hvcmd.ps1 [start|stop|pause|save|stop|shutdown|snapshot] [machinename]
#
# If you would like to start the script via task, use this commandline for example:
# powershell d:\hvcmd.ps1 "pause srv01"


#Getting Parameters from commandline

# UQ - KJD - added $script as we call this from VC from a cmd file wrapper
# Param($operation, $servername)   
  Param($script, $operation, $servername) 

# UQ - KJD - Write Operation and Servername to Output to document results back to VC
Write-Output "Operation		:	$Operation"
Write-Output "Servername	:	$Servername"

#Getting initial VM session names
  $Vm = Get-WmiObject -Namespace root\virtualization  -Query "Select * From Msvm_ComputerSystem Where ElementName = '$servername'"
  $VM_Service = get-wmiobject -namespace root\virtualization Msvm_VirtualSystemManagementService
  $ListofVMs = get-wmiobject -namespace root\virtualization Msvm_ComputerSystem -filter  ElementName <> Name 


#If there is no machine with the desired name, display a warning message
  if ($VM.ElementName -ne $servername)
  {
    write-warning "Virtual machine does not exist!"
    break
  }
	  
#Setting the "state" for later actions	  
  if ($operation -eq 'start') 
  {
    $state = "started"
    $stateId = 2
  }
  elseif ($operation -eq 'stop') 
  {
    $state = "stopped"
    $stateId = 3
  }
  elseif ($operation -eq 'pause') 
  {
    $state = "paused"
	$stateId = 32768
  }
  elseif ($operation -eq 'save') 
  {
    $state = "saved"
    $stateId = 32769
  }
  elseif (($operation -eq 'shutdown') -and ($servername -ne $NUL))
  {
    #$VM_Service = get-wmiobject -namespace root\virtualization Msvm_VirtualSystemManagementService
    $ListofVMs = get-wmiobject -namespace root\virtualization Msvm_ComputerSystem -filter  ElementName <> Name 
    $Vm = Get-WmiObject -Namespace root\virtualization  -Query "Select * From Msvm_ComputerSystem Where ElementName = '$servername'"
    $ShutdownIC = Get-WmiObject -Namespace root\virtualization  -Query "Associators of {$Vm} Where AssocClass=Msvm_SystemDevice ResultClass=Msvm_ShutdownComponent"
    $ShutdownIC.InitiateShutdown("TRUE", "Automated shutdown via script")	
    $state = "shutdown initiated"
	#Do this break because of avoiding errors - not fine but working
    break	
  }
  elseif (($operation -eq 'snapshot')  -and ($servername -ne $NUL))
  {
    #This part took a long time to get it to work, since the state "snapshot" is not recognized, or
    #let's say - wrong recognized in some cases - the state-id "32771" did not work properly for us
    #$VM_Service = get-wmiobject -namespace root\virtualization Msvm_VirtualSystemManagementService
    #$ListofVMs = get-wmiobject -namespace root\virtualization Msvm_ComputerSystem -filter  ElementName <> Name 

    foreach ($VM in [array] $ListOfVMs)
    {
      # $VM.ElementName 
      # $VM.__PATH
      IF ($VM.ElementName -eq $servername)
      {  
        $VM_service.CreateVirtualSystemSnapShot($VM.__PATH)
	    $state = "snapshot initiated"
        $stateId = (32771)
       }  
    }
      $servername + " " + $state
	  break
  }

#Check if specified command is valid
  else 
  {   
	write-warning "Unknown command."
    "Syntax: .\hvcmd.ps1 [command] [machinename]" 
    "Commands: start stop pause save stop shutdown snapshot"
    "Machinename: Name of the Virtual instance (for example: server01)"
	break 
}


#Get a handle to the VM object
  $Core = get-wmiobject -namespace root\virtualization -class Msvm_Computersystem -filter "ElementName = '$servername'" 

#Set the state
  $status = $Core.RequestStateChange($stateId) 

#Actual state
 if ($status.ReturnValue -ne 32775) 
 {
   $servername + " " + $state
 }
  elseif ($status.ReturnValue -eq 32775)
  {    
    "Nothing changed - allready in this state."  
  } 


